/*
 * deviced
 *
 * Copyright (c) 2012 - 2013 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef __DD_HAPTIC_H__
#define __DD_HAPTIC_H__

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file        dd-haptic.h
 * @defgroup    CAPI_SYSTEM_DEVICED_HAPTIC_MODULE Haptic
 * @ingroup     CAPI_SYSTEM_DEVICED
 * @brief       This file provides the API for control of haptic
 * @section CAPI_SYSTEM_DEVICED_HAPTI_MODULE_HEADER Required Header
 *   \#include <dd-haptic.h>
 */

/**
 * @addtogroup CAPI_SYSTEM_DEVICED_HAPTIC_MODULE
 * @{
 */

/**
 * @brief The handle of haptic device
 */
typedef int haptic_device_h;

/**
 * @brief The handle of haptic effect
 */
typedef int haptic_effect_h;

/**
 * @brief Enumerations of device id for the Haptic API.
 * @details We support two motors now.
 */
typedef enum {
	HAPTIC_DEVICE_0 = 0x0,			/**< 1st motor */
	HAPTIC_DEVICE_1 = 0x1,			/**< 2nd motor */
	HAPTIC_DEVICE_ALL = 0x4,		/**< both of them */
} haptic_device_e;

/**
 * @brief Enumerations of priority level for the Haptic API.
 */
typedef enum {
	HAPTIC_PRIORITY_MIN = 0,		/**< Minimum effect priority for developers (default) */
	HAPTIC_PRIORITY_MIDDLE,			/**< Maximum effect priority for developers */
	HAPTIC_PRIORITY_HIGH,			/**< Maximum effect priority for OEMs */
} haptic_priority_e;

/**
 * @brief Enumerations of feedback level for the Haptic API.
 * @details Haptic level means vibration power (intensity).
 */
typedef enum {
	HAPTIC_FEEDBACK_0 = 0,			/**< feedback level 0 */
	HAPTIC_FEEDBACK_1 = 20,			/**< feedback level 1 */
	HAPTIC_FEEDBACK_2 = 40,			/**< feedback level 2 */
	HAPTIC_FEEDBACK_3 = 60,			/**< feedback level 3 */
	HAPTIC_FEEDBACK_4 = 80,			/**< feedback level 4 */
	HAPTIC_FEEDBACK_5 = 100,			/**< feedback level 5 */
	HAPTIC_FEEDBACK_AUTO,			/**< feedback level auto */
} haptic_feedback_e;

/**
 * @brief Enumerations of unlimited duration for the Haptic API.
 */
typedef enum {
	HAPTIC_DURATION_UNLIMITED = 0x7FFFFFFF,
} haptic_duration_e;

/**
 * @brief Enumerations of iteration count for the Haptic API.
 */
typedef enum {
	HAPTIC_ITERATION_ONCE = 1,
	HAPTIC_ITERATION_INFINITE = 256,
} haptic_iteration_e;

/**
 * @brief Enumerations of effect or device state for the Haptic API.
 */
typedef enum {
	HAPTIC_STATE_STOP = 0,
	HAPTIC_STATE_PLAYING,
} haptic_state_e;

/**
 * @brief Enumerations of error codes for the Haptic API.
 */
typedef enum {
	HAPTIC_ERROR_NONE                 = 0,	//TIZEN_ERROR_NONE,			/**< Successful */
	HAPTIC_ERROR_INVALID_PARAMETER    = -1, //TIZEN_ERROR_INVALID_PARAMETER,	/**< Invalid parameter */
	HAPTIC_ERROR_FILE_EXISTS          = -2, //TIZEN_ERROR_FILE_EXISTS,		/**< File exists */
	HAPTIC_ERROR_NOT_INITIALIZED      = -3, //TIZEN_ERROR_SYSTEM_CLASS | 0x26,	/**< Not initialized */
	HAPTIC_ERROR_OPERATION_FAILED     = -4, //TIZEN_ERROR_SYSTEM_CLASS | 0x28,	/**< Operation failed */
	HAPTIC_ERROR_NOT_SUPPORTED_DEVICE = -5, //TIZEN_ERROR_SYSTEM_CLASS | 0x30,	/**< Not supported device */
} haptic_error_e;

/**
 * @brief Gets the number of the vibrators.
 *
 * @remarks The index HAPTIC_DEVICE_ALL is reserved meaning for all vibrators at a time.
 *
 * @param[in] device_number A number of vibrators
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #HAPTIC_ERROR_NONE                   Successful
 * @retval #HAPTIC_ERROR_INVALID_PARAMETER      Invalid parameter
 * @retval #HAPTIC_ERROR_OPERATION_FAILED       Operation failed
 * @retval #HAPTIC_ERROR_NOT_SUPPORTED_DEVICE   Not supported device
 */
int haptic_get_count(int *device_number);

/**
 * @brief Opens a haptic-vibration device.
 *
 * @details Internally, it makes a connection to the vibrator.
 *
 * @remarks If this function is not called in advance, other functions will return #HAPTIC_ERROR_NOT_INITIALIZED.
 * @remarks Haptic API must be closed by haptic_close().
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #HAPTIC_ERROR_NONE                   Successful
 * @retval #HAPTIC_ERROR_INVALID_PARAMETER      Invalid parameter
 * @retval #HAPTIC_ERROR_OPERATION_FAILED       Operation failed
 * @retval #HAPTIC_ERROR_NOT_SUPPORTED_DEVICE   Not supported device
 *
 * @see haptic_close()
 */
int haptic_open(haptic_device_e device, haptic_device_h *device_handle);

/**
 * @brief Closes a haptic-vibration device.
 *
 * @details Internally, it disconnects the connection to vibrator.
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #HAPTIC_ERROR_NONE                   Successful
 * @retval #HAPTIC_ERROR_INVALID_PARAMETER      Invalid parameter
 * @retval #HAPTIC_ERROR_NOT_INITIALIZED        Not initialized
 * @retval #HAPTIC_ERROR_OPERATION_FAILED       Operation failed
 * @retval #HAPTIC_ERROR_NOT_SUPPORTED_DEVICE   Not supported device
 *
 * @see haptic_open()
 */
int haptic_close(haptic_device_h device_handle);

/**
 * @brief Vibrates during the specified time with a constant intensity.
 * @details
 * This function can be used to start monotonous vibration for specified time.
 *
 * @remark
 * If you don't use th api regarding effect_handle, you can pass in a NULL value to last parameter.\n
 * And default value of feedback and priority is used.\n
 * feedback level is reserved for auto chaning to save variable in the settings.\n
 * priority level uses HAPTIC_PRIORITY_MIN.
 *
 * @param[in] device_handle     The device handle from haptic_open()
 * @param[in] duration          The play duration in milliseconds
 * @param[out] effect_handle    Pointer to the variable that will receive a handle to the playing effect
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #HAPTIC_ERROR_NONE                   Successful
 * @retval #HAPTIC_ERROR_INVALID_PARAMETER      Invalid parameter
 * @retval #HAPTIC_ERROR_NOT_INITIALIZED        Not initialized
 * @retval #HAPTIC_ERROR_OPERATION_FAILED       Operation failed
 * @retval #HAPTIC_ERROR_NOT_SUPPORTED_DEVICE   Not supported device
 *
 * @see haptic_vibrate_monotone_with_detail()
 * @see haptic_vibrate_file()
 * @see haptic_vibrate_buffer()
 * @see haptic_get_count()
 */
int haptic_vibrate_monotone(haptic_device_h device_handle, int duration, haptic_effect_h *effect_handle);

/**
 * @brief Stops all vibration effects which are being played.
 * @details This function can be used to stop all effects started by haptic_vibrate_xxx().
 *
 * @remark
 *
 * @param[in] device_handle     The device handle from haptic_open()
 *
 * @return 0 on success, otherwise a negative error value.
 * @retval #HAPTIC_ERROR_NONE                   Successful
 * @retval #HAPTIC_ERROR_INVALID_PARAMETER      Invalid parameter
 * @retval #HAPTIC_ERROR_NOT_INITIALIZED        Not initialized
 * @retval #HAPTIC_ERROR_OPERATION_FAILED       Operation failed
 * @retval #HAPTIC_ERROR_NOT_SUPPORTED_DEVICE   Not supported device
 *
 * @see haptic_vibrate_buffer()
 * @see haptic_vibrate_monotone()
 * @see haptic_vibrate_file()
 * @see haptic_get_count()
 * @see haptic_stop_effect()
 */
int haptic_stop_all_effects(haptic_device_h device_handle);
/**
 * @} end of CAPI_SYSTEM_DEVICED_HAPTIC_MODULE
 */

#ifdef __cplusplus
}
#endif
#endif
